--------------------------------------------------------------------------------
-- ReSynth4
--
-- Copyright 2012 Martin Bealby
--
-- Support code
--------------------------------------------------------------------------------



--------------------------------------------------------------------------------
-- Variables
--------------------------------------------------------------------------------
local env_popup_instance


--------------------------------------------------------------------------------
-- Functions
--------------------------------------------------------------------------------
function clamp(value, min, max)
  -- Clamps a value to a certain range
  return math.min(math.max(min, value), max)
end


function table_to_string(table)
  -- Convert a table of values to a string for storing data in sample names
  
  local str = ''
  
  for i = 1, #table do
    str = str .. tostring(table[i]) .. ':'
  end
  
  -- remove extra ':' from end
  return string.sub(str, 1, string.len(str)-1)
end


function string_to_table(string)
  -- Convert a string to a table by splitting on ':'
  
  -- Code taken from http://www.wellho.net/resources/ex.php4?item=u108/split
  -- and modified
  
  local result = {}
  local from = 1
  local delim_from, delim_to = string.find(string, ':', from)
  while delim_from do
    table.insert(result, string.sub(string, from, delim_from-1))
    from = delim_to + 1
    delim_from, delim_to = string.find(string, ':', from)
  end
  table.insert(result, string.sub(string, from))
  return result
end

function basename(filename)
  -- Convert a full filename into a basename

  local result = {}
  local from = 1
  local delim_from, delim_to = string.find(filename, '[/\\]', from)
  while delim_from do
    table.insert(result, string.sub(filename, from, delim_from-1))
    from = delim_to + 1
    delim_from, delim_to = string.find(filename, '[/\\]', from)
  end
  table.insert(result, string.sub(filename, from))
  
  return(result[#result])
end


function key_passthrough(dialog, key)
  -- Key passthrough function for dialog boxes
  return key
end


function show_env_diagram_popup()
  -- Makes the diagram popup window
  
  if env_popup_instance then
    if env_popup_instance.visible == true then
      env_popup_instance:show()
      return
    end
  end
  
  local vb = renoise.ViewBuilder()
  local env_popup_view
  
  env_popup_instance = nil
  
  env_popup_view = vb:bitmap {
    mode = 'body_color',
    bitmap = 'Images/env4.png',
    tooltip = 'AHDBDSR envelope reference',
  }
  
  env_popup_instance = renoise.app():show_custom_dialog('Envelope Reference',
                                                        env_popup_view)
  
end


--------------------------------------------------------------------------------
-- Wavedata
--------------------------------------------------------------------------------
SIN_TABLE = {
  -- first sine wave
  0.031410759078128,
  0.062790519529313,
  0.094108313318514,
  0.1253332335643,
  0.15643446504023,
  0.18738131458572,
  0.21814324139654,
  0.24868988716485,
  0.27899110603923,
  0.30901699437495,
  0.33873792024529,
  0.36812455268468,
  0.39714789063478,
  0.42577929156507,
  0.45399049973955,
  0.48175367410172,
  0.50904141575037,
  0.535826794979,
  0.56208337785213,
  0.58778525229247,
  0.61290705365298,
  0.63742398974869,
  0.66131186532365,
  0.68454710592869,
  0.70710678118655,
  0.72896862742141,
  0.75011106963046,
  0.77051324277579,
  0.79015501237569,
  0.80901699437495,
  0.82708057427456,
  0.84432792550202,
  0.86074202700394,
  0.87630668004386,
  0.89100652418837,
  0.90482705246602,
  0.91775462568398,
  0.92977648588825,
  0.94088076895423,
  0.95105651629515,
  0.96029368567694,
  0.96858316112863,
  0.97591676193875,
  0.98228725072869,
  0.98768834059514,
  0.99211470131448,
  0.99556196460308,
  0.99802672842827,
  0.99950656036573,
  1.00000000000000,
  0.99950656036573,
  0.99802672842827,
  0.99556196460308,
  0.99211470131448,
  0.98768834059514,
  0.98228725072869,
  0.97591676193875,
  0.96858316112863,
  0.96029368567694,
  0.95105651629515,
  0.94088076895423,
  0.92977648588825,
  0.91775462568398,
  0.90482705246602,
  0.89100652418837,
  0.87630668004386,
  0.86074202700394,
  0.84432792550201,
  0.82708057427456,
  0.80901699437495,
  0.79015501237569,
  0.77051324277579,
  0.75011106963046,
  0.72896862742141,
  0.70710678118655,
  0.68454710592869,
  0.66131186532365,
  0.63742398974869,
  0.61290705365298,
  0.58778525229247,
  0.56208337785213,
  0.53582679497900,
  0.50904141575037,
  0.48175367410172,
  0.45399049973955,
  0.42577929156507,
  0.39714789063478,
  0.36812455268468,
  0.33873792024529,
  0.30901699437495,
  0.27899110603923,
  0.24868988716486,
  0.21814324139654,
  0.18738131458572,
  0.15643446504023,
  0.12533323356430,
  0.09410831331851,
  0.06279051952931,
  0.03141075907812,
  0.00000000000000,
  -0.03141075907812,
  -0.06279051952931,
  -0.09410831331851,
  -0.12533323356430,
  -0.15643446504023,
  -0.18738131458572,
  -0.21814324139654,
  -0.24868988716485,
  -0.27899110603923,
  -0.30901699437495,
  -0.33873792024529,
  -0.36812455268468,
  -0.39714789063478,
  -0.42577929156507,
  -0.45399049973955,
  -0.48175367410172,
  -0.50904141575037,
  -0.53582679497900,
  -0.56208337785213,
  -0.58778525229247,
  -0.61290705365298,
  -0.63742398974869,
  -0.66131186532365,
  -0.68454710592869,
  -0.70710678118655,
  -0.72896862742141,
  -0.75011106963046,
  -0.77051324277579,
  -0.79015501237569,
  -0.80901699437495,
  -0.82708057427456,
  -0.84432792550202,
  -0.86074202700394,
  -0.87630668004386,
  -0.89100652418837,
  -0.90482705246602,
  -0.91775462568398,
  -0.92977648588825,
  -0.94088076895423,
  -0.95105651629515,
  -0.96029368567694,
  -0.96858316112863,
  -0.97591676193875,
  -0.98228725072869,
  -0.98768834059514,
  -0.99211470131448,
  -0.99556196460308,
  -0.99802672842827,
  -0.99950656036573,
  -1.00000000000000,
  -0.99950656036573,
  -0.99802672842827,
  -0.99556196460308,
  -0.99211470131448,
  -0.98768834059514,
  -0.98228725072869,
  -0.97591676193875,
  -0.96858316112863,
  -0.96029368567694,
  -0.95105651629515,
  -0.94088076895423,
  -0.92977648588825,
  -0.91775462568398,
  -0.90482705246602,
  -0.89100652418837,
  -0.87630668004386,
  -0.86074202700394,
  -0.84432792550201,
  -0.82708057427456,
  -0.80901699437495,
  -0.79015501237569,
  -0.77051324277579,
  -0.75011106963046,
  -0.72896862742141,
  -0.70710678118655,
  -0.68454710592869,
  -0.66131186532365,
  -0.63742398974869,
  -0.61290705365298,
  -0.58778525229247,
  -0.56208337785213,
  -0.53582679497900,
  -0.50904141575037,
  -0.48175367410172,
  -0.45399049973955,
  -0.42577929156507,
  -0.39714789063478,
  -0.36812455268468,
  -0.33873792024529,
  -0.30901699437495,
  -0.27899110603923,
  -0.24868988716486,
  -0.21814324139654,
  -0.18738131458572,
  -0.15643446504023,
  -0.12533323356430,
  -0.09410831331851,
  -0.06279051952931,
  -0.03141075907812,
  0.00000000000000,
  
  -- second sine wave
  0.031410759078128,
  0.062790519529313,
  0.094108313318514,
  0.1253332335643,
  0.15643446504023,
  0.18738131458572,
  0.21814324139654,
  0.24868988716485,
  0.27899110603923,
  0.30901699437495,
  0.33873792024529,
  0.36812455268468,
  0.39714789063478,
  0.42577929156507,
  0.45399049973955,
  0.48175367410172,
  0.50904141575037,
  0.535826794979,
  0.56208337785213,
  0.58778525229247,
  0.61290705365298,
  0.63742398974869,
  0.66131186532365,
  0.68454710592869,
  0.70710678118655,
  0.72896862742141,
  0.75011106963046,
  0.77051324277579,
  0.79015501237569,
  0.80901699437495,
  0.82708057427456,
  0.84432792550202,
  0.86074202700394,
  0.87630668004386,
  0.89100652418837,
  0.90482705246602,
  0.91775462568398,
  0.92977648588825,
  0.94088076895423,
  0.95105651629515,
  0.96029368567694,
  0.96858316112863,
  0.97591676193875,
  0.98228725072869,
  0.98768834059514,
  0.99211470131448,
  0.99556196460308,
  0.99802672842827,
  0.99950656036573,
  1.00000000000000,
  0.99950656036573,
  0.99802672842827,
  0.99556196460308,
  0.99211470131448,
  0.98768834059514,
  0.98228725072869,
  0.97591676193875,
  0.96858316112863,
  0.96029368567694,
  0.95105651629515,
  0.94088076895423,
  0.92977648588825,
  0.91775462568398,
  0.90482705246602,
  0.89100652418837,
  0.87630668004386,
  0.86074202700394,
  0.84432792550201,
  0.82708057427456,
  0.80901699437495,
  0.79015501237569,
  0.77051324277579,
  0.75011106963046,
  0.72896862742141,
  0.70710678118655,
  0.68454710592869,
  0.66131186532365,
  0.63742398974869,
  0.61290705365298,
  0.58778525229247,
  0.56208337785213,
  0.53582679497900,
  0.50904141575037,
  0.48175367410172,
  0.45399049973955,
  0.42577929156507,
  0.39714789063478,
  0.36812455268468,
  0.33873792024529,
  0.30901699437495,
  0.27899110603923,
  0.24868988716486,
  0.21814324139654,
  0.18738131458572,
  0.15643446504023,
  0.12533323356430,
  0.09410831331851,
  0.06279051952931,
  0.03141075907812,
  0.00000000000000,
  -0.03141075907812,
  -0.06279051952931,
  -0.09410831331851,
  -0.12533323356430,
  -0.15643446504023,
  -0.18738131458572,
  -0.21814324139654,
  -0.24868988716485,
  -0.27899110603923,
  -0.30901699437495,
  -0.33873792024529,
  -0.36812455268468,
  -0.39714789063478,
  -0.42577929156507,
  -0.45399049973955,
  -0.48175367410172,
  -0.50904141575037,
  -0.53582679497900,
  -0.56208337785213,
  -0.58778525229247,
  -0.61290705365298,
  -0.63742398974869,
  -0.66131186532365,
  -0.68454710592869,
  -0.70710678118655,
  -0.72896862742141,
  -0.75011106963046,
  -0.77051324277579,
  -0.79015501237569,
  -0.80901699437495,
  -0.82708057427456,
  -0.84432792550202,
  -0.86074202700394,
  -0.87630668004386,
  -0.89100652418837,
  -0.90482705246602,
  -0.91775462568398,
  -0.92977648588825,
  -0.94088076895423,
  -0.95105651629515,
  -0.96029368567694,
  -0.96858316112863,
  -0.97591676193875,
  -0.98228725072869,
  -0.98768834059514,
  -0.99211470131448,
  -0.99556196460308,
  -0.99802672842827,
  -0.99950656036573,
  -1.00000000000000,
  -0.99950656036573,
  -0.99802672842827,
  -0.99556196460308,
  -0.99211470131448,
  -0.98768834059514,
  -0.98228725072869,
  -0.97591676193875,
  -0.96858316112863,
  -0.96029368567694,
  -0.95105651629515,
  -0.94088076895423,
  -0.92977648588825,
  -0.91775462568398,
  -0.90482705246602,
  -0.89100652418837,
  -0.87630668004386,
  -0.86074202700394,
  -0.84432792550201,
  -0.82708057427456,
  -0.80901699437495,
  -0.79015501237569,
  -0.77051324277579,
  -0.75011106963046,
  -0.72896862742141,
  -0.70710678118655,
  -0.68454710592869,
  -0.66131186532365,
  -0.63742398974869,
  -0.61290705365298,
  -0.58778525229247,
  -0.56208337785213,
  -0.53582679497900,
  -0.50904141575037,
  -0.48175367410172,
  -0.45399049973955,
  -0.42577929156507,
  -0.39714789063478,
  -0.36812455268468,
  -0.33873792024529,
  -0.30901699437495,
  -0.27899110603923,
  -0.24868988716486,
  -0.21814324139654,
  -0.18738131458572,
  -0.15643446504023,
  -0.12533323356430,
  -0.09410831331851,
  -0.06279051952931,
  -0.03141075907812,
  0.00000000000000
}


SAW_TABLE = {
  0.01,
  0.02,
  0.03,
  0.04,
  0.05,
  0.06,
  0.07,
  0.08,
  0.09,
  0.10,
  0.11,
  0.12,
  0.13,
  0.14,
  0.15,
  0.16,
  0.17,
  0.18,
  0.19,
  0.20,
  0.21,
  0.22,
  0.23,
  0.24,
  0.25,
  0.26,
  0.27,
  0.28,
  0.29,
  0.30,
  0.31,
  0.32,
  0.33,
  0.34,
  0.35,
  0.36,
  0.37,
  0.38,
  0.39,
  0.40,
  0.41,
  0.42,
  0.43,
  0.44,
  0.45,
  0.46,
  0.47,
  0.48,
  0.49,
  0.50,
  0.51,
  0.52,
  0.53,
  0.54,
  0.55,
  0.56,
  0.57,
  0.58,
  0.59,
  0.60,
  0.61,
  0.62,
  0.63,
  0.64,
  0.65,
  0.66,
  0.67,
  0.68,
  0.69,
  0.70,
  0.71,
  0.72,
  0.73,
  0.74,
  0.75,
  0.76,
  0.77,
  0.78,
  0.79,
  0.80,
  0.81,
  0.82,
  0.83,
  0.84,
  0.85,
  0.86,
  0.87,
  0.88,
  0.89,
  0.90,
  0.91,
  0.92,
  0.93,
  0.94,
  0.95,
  0.96,
  0.97,
  0.98,
  0.99,
  1.00,
  -0.99,
  -0.98,
  -0.97,
  -0.96,
  -0.95,
  -0.94,
  -0.93,
  -0.92,
  -0.91,
  -0.90,
  -0.89,
  -0.88,
  -0.87,
  -0.86,
  -0.85,
  -0.84,
  -0.83,
  -0.82,
  -0.81,
  -0.80,
  -0.79,
  -0.78,
  -0.77,
  -0.76,
  -0.75,
  -0.74,
  -0.73,
  -0.72,
  -0.71,
  -0.70,
  -0.69,
  -0.68,
  -0.67,
  -0.66,
  -0.65,
  -0.64,
  -0.63,
  -0.62,
  -0.61,
  -0.60,
  -0.59,
  -0.58,
  -0.57,
  -0.56,
  -0.55,
  -0.54,
  -0.53,
  -0.52,
  -0.51,
  -0.50,
  -0.49,
  -0.48,
  -0.47,
  -0.46,
  -0.45,
  -0.44,
  -0.43,
  -0.42,
  -0.41,
  -0.40,
  -0.39,
  -0.38,
  -0.37,
  -0.36,
  -0.35,
  -0.34,
  -0.33,
  -0.32,
  -0.31,
  -0.30,
  -0.29,
  -0.28,
  -0.27,
  -0.26,
  -0.25,
  -0.24,
  -0.23,
  -0.22,
  -0.21,
  -0.20,
  -0.19,
  -0.18,
  -0.17,
  -0.16,
  -0.15,
  -0.14,
  -0.13,
  -0.12,
  -0.11,
  -0.10,
  -0.09,
  -0.08,
  -0.07,
  -0.06,
  -0.05,
  -0.04,
  -0.03,
  -0.02,
  -0.01,
  0.00
}
